﻿Imports System.Speech

' 音声認識エンジンを使って、VISA測定器を制御する。
' 2015/10/10 7M4MON
' 2016/01/06 色でコマンドを発行する機能を削除した

Public Class Form1
    Dim creditStr As String = "(C) 7M4MON 2015"
    Dim WithEvents speechRecog As New Recognition.SpeechRecognitionEngine

    Private Sub SetColor(ByVal color As System.Drawing.Color)

        ToolStripStatusLabel1.BackColor = color

    End Sub

    Dim dictionaryArrayStr() As String
    Dim colorArrayStr() As String = {"black", "brown", "red", "orange", "yellow", "green", "blue", "purple", "gray", "white"}
    '辞書を構築する
    Private Sub BuildDictionary()
        'ボイスコマンドのテキストボックスかの判別
        Dim dictionaryArrayIndex As Integer = 0
        dictionaryArrayStr = {vc0Tb.Text, vc1Tb.Text, vc2Tb.Text, vc3Tb.Text, vc4Tb.Text, vc5Tb.Text, vc6Tb.Text, vc7Tb.Text, vc8Tb.Text, vc9Tb.Text}

        For Each vaTbCtrl As Control In Me.Controls

            'ボイスコマンドのテキストボックスかの判別
            If TypeOf vaTbCtrl Is TextBox And vaTbCtrl.Name.IndexOf("vc") > -1 Then
                '対応する辞書配列にテキストボックスの中身を入れる
                dictionaryArrayIndex = CInt(vaTbCtrl.Name.Substring(2, 1))
                dictionaryArrayStr(dictionaryArrayIndex) = vaTbCtrl.Text

                '空だったら代わりに数字を入れる
                If dictionaryArrayStr(dictionaryArrayIndex) = "" Then
                    dictionaryArrayStr(dictionaryArrayIndex) = CStr(dictionaryArrayIndex)
                End If

            End If
        Next
    End Sub

    '音声認識の初期化
    Private Sub InitializeRecognizer()

        'マイクの選択
        speechRecog.SetInputToDefaultAudioDevice()

        '辞書のロード
        LoadGrammer()

        '音声認識の開始
        speechRecog.RecognizeAsync()

    End Sub

    '辞書をロードする関数
    Private Sub LoadGrammer()
        Dim gram As New Recognition.SrgsGrammar.SrgsDocument
        Dim commandRule As New Recognition.SrgsGrammar.SrgsRule("command")
        Dim commandList As New Recognition.SrgsGrammar.SrgsOneOf(dictionaryArrayStr)

        commandRule.Add(commandList)
        gram.Rules.Add(commandRule)
        gram.Root = commandRule
        'gram.Culture = New Globalization.CultureInfo(cultureInfoStr)
        speechRecog.LoadGrammar(New Recognition.Grammar(gram))

        '' 普通の文字認識の場合は下記辞書を使う
        'speechRecog.LoadGrammar(New Recognition.DictationGrammar())
    End Sub
    Private Sub UnLoadGrammer()
        speechRecog.UnloadAllGrammars()
    End Sub
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        '保存された設定のロード
        LoadSettings()

        '現在の言語設定の取得
        Dim cultureInfoStr As String
        cultureInfoStr = Globalization.CultureInfo.CurrentCulture.TwoLetterISOLanguageName
        Me.Text += "  Speech Language : " & cultureInfoStr

        BuildDictionary()

        InitializeRecognizer()
        Me.ActiveControl = Me.cs1Tb     '最初にva1Tbにフォーカスがあると移動した時に必ず　Voice Command Updated　が出てしまうのが気持ち悪かった。
        infomationLbl.Text = creditStr

    End Sub

    'Recognizedが完了した後に発生するイベント。
    'speechRecog.RecognizeAsync() で音声認識を再び開始する。
    Private Sub reco_RecognizeCompleted(ByVal sender As Object, ByVal e As System.Speech.Recognition.RecognizeCompletedEventArgs) Handles speechRecog.RecognizeCompleted

        speechRecog.RecognizeAsync()

    End Sub

    '音声が入力されたときの処理。
    Private Sub reco_SpeechRecognized(ByVal sender As Object, ByVal e As System.Speech.Recognition.RecognitionEventArgs) Handles speechRecog.SpeechRecognized

        Dim recoResultStr As String
        recoResultStr = e.Result.Text
        Debug.Print(recoResultStr)
        Dim dictinaryIndex As Integer = Array.IndexOf(dictionaryArrayStr, recoResultStr)
        BuildCommand(dictinaryIndex)

    End Sub

    Private Sub LoadSettings()
        Try
            With My.Settings
                vc1Tb.Text = .vc1
                vc2Tb.Text = .vc2
                vc3Tb.Text = .vc3
                vc4Tb.Text = .vc4
                vc5Tb.Text = .vc5
                vc6Tb.Text = .vc6
                vc7Tb.Text = .vc7
                vc8Tb.Text = .vc8
                vc9Tb.Text = .vc9
                vc0Tb.Text = .vc0
                va1Tb.Text = .va1
                va2Tb.Text = .va2
                va3Tb.Text = .va3
                va4Tb.Text = .va4
                va5Tb.Text = .va5
                va6Tb.Text = .va6
                va7Tb.Text = .va7
                va8Tb.Text = .va8
                va9Tb.Text = .va9
                va0Tb.Text = .va0
                cs1Tb.Text = .cs1
                cs2Tb.Text = .cs2
                cs3Tb.Text = .cs3
                cs4Tb.Text = .cs4
                cs5Tb.Text = .cs5
                cs6Tb.Text = .cs6
                cs7Tb.Text = .cs7
                cs8Tb.Text = .cs8
                cs9Tb.Text = .cs9
                cs0Tb.Text = .cs0
            End With
        Catch ex As Exception

        End Try

    End Sub
    Private Sub SaveSettings()
        Try
            With My.Settings
                .vc1 = vc1Tb.Text
                .vc2 = vc2Tb.Text
                .vc3 = vc3Tb.Text
                .vc4 = vc4Tb.Text
                .vc5 = vc5Tb.Text
                .vc6 = vc6Tb.Text
                .vc7 = vc7Tb.Text
                .vc8 = vc8Tb.Text
                .vc9 = vc9Tb.Text
                .vc0 = vc0Tb.Text
                .va1 = va1Tb.Text
                .va2 = va2Tb.Text
                .va3 = va3Tb.Text
                .va4 = va4Tb.Text
                .va5 = va5Tb.Text
                .va6 = va6Tb.Text
                .va7 = va7Tb.Text
                .va8 = va8Tb.Text
                .va9 = va9Tb.Text
                .va0 = va0Tb.Text
                .cs1 = cs1Tb.Text
                .cs2 = cs2Tb.Text
                .cs3 = cs3Tb.Text
                .cs4 = cs4Tb.Text
                .cs5 = cs5Tb.Text
                .cs6 = cs6Tb.Text
                .cs7 = cs7Tb.Text
                .cs8 = cs8Tb.Text
                .cs9 = cs9Tb.Text
                .cs0 = cs0Tb.Text
            End With
        Catch ex As Exception

        End Try
    End Sub

    'フォームが閉じる時に設定を保存する
    Private Sub Form1_FormClosing(sender As System.Object, e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        SaveSettings()
    End Sub

    'テスト用に色の■ラベルをクリックしたらコマンドを発動するようにした。
    Private Sub VcLabel_Click(sender As System.Object, e As System.EventArgs) Handles vc1Lbl.Click, vc2Lbl.Click, vc3Lbl.Click, vc4Lbl.Click, vc5Lbl.Click, _
                                                                                        vc6Lbl.Click, vc7Lbl.Click, vc8Lbl.Click, vc9Lbl.Click, vc10Lbl.Click
        Dim lblColor As Color
        lblColor = sender.ForeColor
        Dim lblColorStr As String
        lblColorStr = lblColor.Name.ToLower
        Dim lblColorIndex As Integer = Array.IndexOf(colorArrayStr, lblColorStr)
        BuildCommand(lblColorIndex)

    End Sub

    '特殊文字の置換
    Private Function ReplaceDateNow(beforeStr As String) As String
        Dim afterStr As String
        afterStr = beforeStr.Replace("%NOW%", Date.Now.ToString("HHmmss"))
        afterStr = afterStr.Replace("%TODAY%", Date.Now.ToString("yyyyMMdd"))
        afterStr = afterStr.Replace("%LF%", vbLf)
        afterStr = afterStr.Replace("%CR%", vbCr)
        ReplaceDateNow = afterStr
    End Function

    '音声コマンドから、VISAコマンドを発行
    '2016/01/06 色を引数にするのではなく、数値を引数にするようにした。
    Private Sub BuildCommand(aryIndex As Integer)

        infomationLbl.Text = dictionaryArrayStr(aryIndex)   'デバッグ用の表示

        aryIndex = aryIndex Mod 10
        Debug.Print(aryIndex)
        Select Case aryIndex
            Case 1
                SetColor(Color.Brown)
                Application.DoEvents()
                SendCommand(va1Tb.Text, cs1Tb.Text)
            Case 2
                SetColor(Color.Red)
                Application.DoEvents()
                SendCommand(va2Tb.Text, cs2Tb.Text)
            Case 3
                SetColor(Color.Orange)
                Application.DoEvents()
                SendCommand(va3Tb.Text, cs3Tb.Text)
            Case 4
                SetColor(Color.Yellow)
                Application.DoEvents()
                SendCommand(va4Tb.Text, cs4Tb.Text)
            Case 5
                SetColor(Color.Green)
                Application.DoEvents()
                SendCommand(va5Tb.Text, cs5Tb.Text)
            Case 6
                SetColor(Color.Blue)
                Application.DoEvents()
                SendCommand(va6Tb.Text, cs6Tb.Text)
            Case 7
                SetColor(Color.Purple)
                Application.DoEvents()
                SendCommand(va7Tb.Text, cs7Tb.Text)
            Case 8
                SetColor(Color.Gray)
                Application.DoEvents()
                SendCommand(va8Tb.Text, cs8Tb.Text)
            Case 9
                SetColor(Color.White)
                Application.DoEvents()
                SendCommand(va9Tb.Text, cs9Tb.Text)
            Case 0
                SetColor(Color.Black)
                Application.DoEvents()
                SendCommand(va0Tb.Text, cs0Tb.Text)

        End Select
        ' infomationLbl.Text = Now() + " : " + colorStr
    End Sub

    '参照のVISA32.vbから、VISA32.DLLを呼び出してVISAで通信する
    Private Sub SendCommand(visaAddress As String, visaCommandString As String)
        Try
            visaCommandString = ReplaceDateNow(visaCommandString)
            Dim resourceManager As Integer = 0, viError As Integer
            Dim session As Integer = 0
            viError = visa32.viOpenDefaultRM(resourceManager)
            viError = visa32.viOpen(resourceManager, visaAddress, _
                            visa32.VI_NO_LOCK, visa32.VI_TMO_IMMEDIATE, session)
            'Also, since you are using SOCKET connections, you will need to enable the termination character, 
            'or all of your viScanf/viQueryf calls will fail. To do that, add this code after the viOpen calls:
            'viSetAttribute(session, VI_ATTR_TERMCHAR_EN, VI_TRUE)

            viError = visa32.viPrintf(session, visaCommandString & vbLf)

            infomationLbl.Text += ", " + Now.ToString("HH:mm:ss") + ", " + visaAddress + ", " + visaCommandString
            If viError = 0 Then
                infomationLbl.Text = "OK, " + infomationLbl.Text
            Else
                infomationLbl.Text = "NG, " + infomationLbl.Text
            End If
            ' infomationLbl.Text = Now.ToString("HH:mm:ss") + ", " + visaAddress + ", " + visaCommandString + ", " + viError.ToString

            Dim replyString As System.Text.StringBuilder = _
                            New System.Text.StringBuilder(1000)
            viError = visa32.viScanf(session, "%s", replyString)
            replyLbl.Text = replyString.ToString()
            visa32.viClose(session)
            visa32.viClose(resourceManager)

        Catch ex As Exception
            infomationLbl.Text += "ERROR, " + Now.ToString("HH:mm:ss") + ", " + visaAddress + ", " + visaCommandString
        End Try
    End Sub

    '音声認識のコマンドが、まさにいま変えられている時
    Private Sub Tb_TextChanged(sender As System.Object, e As System.EventArgs) Handles vc1Tb.TextChanged, vc2Tb.TextChanged, vc3Tb.TextChanged, vc4Tb.TextChanged, vc5Tb.TextChanged, _
                                                                                        vc6Tb.TextChanged, vc7Tb.TextChanged, vc8Tb.TextChanged, vc9Tb.TextChanged, vc0Tb.TextChanged
        infomationLbl.Text = "Press TAB key to Confirm"
    End Sub

    '音声認識のコマンドを変えて、確定したら辞書の再構築を行う
    Private Sub vc1Tb_Leave(sender As System.Object, e As System.EventArgs) Handles vc1Tb.Leave, vc2Tb.Leave, vc3Tb.Leave, vc4Tb.Leave, vc5Tb.Leave, _
                                                                      vc6Tb.Leave, vc7Tb.Leave, vc8Tb.Leave, vc9Tb.Leave, vc0Tb.Leave
        BuildDictionary()
        UnLoadGrammer()
        LoadGrammer()
        infomationLbl.Text = Now.ToString("HH:mm:ss") + ", Voice Command Updated"
    End Sub

End Class